#!/bin/sh
# Author: Andreas Leitgeb <avl@logic.at> \
exec wish "$0" "$@"

#  You may use this program freely. If you add features, 
# I'd be happy to be notified and sent a copy upon request.

#  "q" quits,  "o" open option-dialog (set volume and duration)
# Side-effects: sets the XTerminal-bell with "xset b <vol> <pitch> <dur>" 

# other variable-initializations
set lastplayed {}
set lastfreq 0
set duration 1000
set volume 100
set timer {}

# frequency-table 
#   (currently, only the integral part is actually used)
set freqscale(72) 8372.018090
set freqscale(71) 7902.132820
set freqscale(70) 7458.620184
set freqscale(69) 7040.000000
set freqscale(68) 6644.875161
set freqscale(67) 6271.926976
set freqscale(66) 5919.910763
set freqscale(65) 5587.651703
set freqscale(64) 5274.040911
set freqscale(63) 4978.031740
set freqscale(62) 4698.636287
set freqscale(61) 4434.922096
set freqscale(60) 4186.009045
set freqscale(59) 3951.066410
set freqscale(58) 3729.310092
set freqscale(57) 3520.000000
set freqscale(56) 3322.437581
set freqscale(55) 3135.963488
set freqscale(54) 2959.955382
set freqscale(53) 2793.825851
set freqscale(52) 2637.020455
set freqscale(51) 2489.015870
set freqscale(50) 2349.318143
set freqscale(49) 2217.461048
set freqscale(48) 2093.004522
set freqscale(47) 1975.533205
set freqscale(46) 1864.655046
set freqscale(45) 1760.000000
set freqscale(44) 1661.218790
set freqscale(43) 1567.981744
set freqscale(42) 1479.977691
set freqscale(41) 1396.912926
set freqscale(40) 1318.510228
set freqscale(39) 1244.507935
set freqscale(38) 1174.659072
set freqscale(37) 1108.730524
set freqscale(36) 1046.502261
set freqscale(35) 987.766603
set freqscale(34) 932.327523
set freqscale(33) 880.000000
set freqscale(32) 830.609395
set freqscale(31) 783.990872
set freqscale(30) 739.988845
set freqscale(29) 698.456463
set freqscale(28) 659.255114
set freqscale(27) 622.253967
set freqscale(26) 587.329536
set freqscale(25) 554.365262
set freqscale(24) 523.251131
set freqscale(23) 493.883301
set freqscale(22) 466.163762
set freqscale(21) 440.000000
set freqscale(20) 415.304698
set freqscale(19) 391.995436
set freqscale(18) 369.994423
set freqscale(17) 349.228231
set freqscale(16) 329.627557
set freqscale(15) 311.126984
set freqscale(14) 293.664768
set freqscale(13) 277.182631
set freqscale(12) 261.625565
set freqscale(11) 246.941651
set freqscale(10) 233.081881
set freqscale(9) 220.000000
set freqscale(8) 207.652349
set freqscale(7) 195.997718
set freqscale(6) 184.997211
set freqscale(5) 174.614116
set freqscale(4) 164.813778
set freqscale(3) 155.563492
set freqscale(2) 146.832384
set freqscale(1) 138.591315
set freqscale(0) 130.812783

# Helper-procedures:
proc quit_program {} {
     exec xset b 100 440 30
     exit
}
proc play f { global volume duration lastfreq timer
     exec xset b $volume $f $duration; set lastfreq $f
     set timer [after $duration [list play $f]]
     bell 
}
proc stopplay {} { global volume lastfreq timer
     exec xset b $volume $lastfreq 1
     bell ; after cancel $timer
}
proc calcfreq {t o} { global freqscale
    return [expr int($freqscale([expr {$t + 12 * $o}]))]
}
proc whitekey {w t p} { set t {}
   label $w -text $t -background white -foreground black \
        -borderwidth 2 -relief raised 
   place $w -relx $p -y 0 -relwidth 0.143 -relheight 1
   bindtags $w {key}
   lower $w
}
proc blackkey {w t p} { set t {}
   label $w -text $t -background black -foreground white \
        -borderwidth 2 -relief raised 
   place $w -relx $p -y 0 -relwidth 0.1 -relheight 0.6 -anchor n
   bindtags $w {key}
}

# Event-handler procedures
proc clickon w { global lastplayed; set lastplayed $w
    $w conf -relief sunken; update idletasks
    play [winfo name $w]
#    $w conf -relief raised; update idletasks
}
proc keyrelease {} { global lastplayed
    if {$lastplayed != {}} {
        $lastplayed conf -relief raised; update idletasks
        stopplay
    }
}
    
proc dragon {x y} { global lastplayed; 
    set w [winfo containing $x $y]
    if { $w == "" } { keyrelease; set lastplayed {}; return }
    if { $w == $lastplayed } return
    if {[lsearch -exact [bindtags $w] key ] == -1 } {
         keyrelease
         set lastplayed {}; return
    } 
    keyrelease
    clickon $w
}

# build the keyboard-window
frame .toplevel -height 100 -width 900; pack .toplevel -fill both -expand yes
for {set o 0} {$o < 6} {incr o} {
   set w .toplevel.f$o
   frame $w -height 100 -width 150
      whitekey $w.[calcfreq 0 $o]  C  0.0
      blackkey $w.[calcfreq 1 $o]  C# 0.143
      whitekey $w.[calcfreq 2 $o]  D  0.143
      blackkey $w.[calcfreq 3 $o]  D# 0.286
      whitekey $w.[calcfreq 4 $o]  E  0.286
      whitekey $w.[calcfreq 5 $o]  F  0.429
      blackkey $w.[calcfreq 6 $o]  F# 0.571
      whitekey $w.[calcfreq 7 $o]  G  0.571
      blackkey $w.[calcfreq 8 $o]  G# 0.714
      whitekey $w.[calcfreq 9 $o]  A  0.714
      blackkey $w.[calcfreq 10 $o] A# 0.857
      whitekey $w.[calcfreq 11 $o] H  0.857
   place $w -y 0 -relheight 1.0 -relx [expr {$o / 6.0}] -relwidth 0.1667 
}
focus .toplevel

# Key- and Mousebindings
bind key <1> {clickon %W}
bind key <B1-Motion> {dragon %X %Y}
bind key <ButtonRelease-1> {keyrelease}

bind .toplevel <q> quit_program
wm protocol . WM_DELETE_WINDOW quit_program
bind .toplevel <o> options

# options-dialog
proc options {} { set w .options
    if {[winfo exists $w ]} {return}
    toplevel $w -class Piano-Options
    frame $w.1 -bd 1 -relief raised
       label $w.1.l -text Volume
       scale $w.1.volume -from 100 -to 0 -sliderlength 10 \
           -variable volume -orient vertical
    pack $w.1.l -fill x 
    pack $w.1.volume -fill both -expand yes

    frame $w.2 -bd 1 -relief raised
       label $w.2.l -text Duration
       scale $w.2.duration -from 1000 -to 0 -sliderlength 10 \
           -variable duration -orient vertical
    pack $w.2.l -fill x
    pack $w.2.duration -fill both -expand yes
    
    frame $w.3 -bd 1 -relief raised
       button $w.3.quit -text Dismiss -command [list destroy $w]
    pack $w.3.quit -padx 5 -pady 5 -fill both -expand yes

    pack $w.3 -side bottom -fill x
    pack $w.1 $w.2 -side left -fill both -expand yes
}

